% This software is offered under the GNU General Public License 3.0.  It 
% is offered without any warranty expressed or implied, including the 
% implied warranties of merchantability or fitness for a particular 
% purpose.

% Written by Nicholas Dwork (ndwork@stanford.edu) and 
% Gennifer Smith (gsmith9@stanford.edu).


function [I, z, dx, z0, zR, alpha, beta, L0, lambda, deltaLambda, ...
  dLambda, noisePower, trueMu ] = loadOctData(dataCase, plotIt, dataDir)

  if nargin<2, plotIt=false; end;

  trueMu = [];
  cols2Del = [];
  rows2Del = [];
  alpha = [];
  beta = [];
  L0 = [];

  [lambda,deltaLambda,dLambda] = getTelestoFalloffParams();
  
  if ispc
      slash = '\';
  else
      slash = '/';
  end
  
  switch dataCase
    case 0
      N = 100;
      dx = 13d-3;
      [I,z,z0,zR,alpha,beta,L0,trueMu] = makePhantom2D(N,5);
      alpha_R = 2;
      n = 1.37;
      noisePower = (0.5d-2)^2;
      return

    case 1 % Intralipid 1.25
      %z0 = 0.75;
      zR = 0.1059;
      dx = 0.00502;
      imgDepth = 2.57;
      numPix = 512;
      datafile = [ dataDir, slash, 'Intralipid_1p25.cvol' ];
      z0 = 1.6;
      rows2Del = [401:numPix];
      alpha_R = 2;
      n = 1.353;
      noisePower = (1d-1)^2;

    case 2 % Intralipid 2.5
      %z0 = 0.65;
      zR = 0.1059;
      dx = 0.00502;
      imgDepth = 2.57;
      numPix = 512;
      datafile = [ dataDir, slash, 'Intralipid_2p5.cvol' ];
      z0 = 1.6;
      rows2Del = [401:numPix];
      alpha_R = 2;
      n = 1.353;
      noisePower = (1d-1)^2;

    case 3 % Intralipid 5
      %z0 = 0.65;
      zR = 0.1059;
      dx = 0.00502;
      imgDepth = 2.57;
      numPix = 512;
      datafile = [ dataDir, slash, 'Intralipid_5.cvol' ];
      z0 = 1.6;
      rows2Del = [401:numPix];
      alpha_R = 2;
      n = 1.353;
      noisePower = (1d-1)^2;

    case 4 % Intralipid 10
      %z0 = 0.65;
      zR = 0.1059;
      dx = 0.00502;
      imgDepth = 2.57;
      numPix = 512;
      datafile = [ dataDir, slash, 'Intralipid_10.cvol' ];
      z0 = 1.6;
      rows2Del = [401:numPix];
      alpha_R = 2;
      n = 1.353;
      fileIndx = 3;
      noisePower = (1d-1)^2;

    case 5 % Intralipid 15
      %z0 = 0.7;
      zR = 0.1059;
      dx = 0.00502;
      imgDepth = 2.57;
      numPix = 512;
      datafile = [ dataDir, slash, 'Intralipid_15.cvol' ];
      z0 = 1.6;
      rows2Del = [401:numPix];
      alpha_R = 2;
      n = 1.353;
      noisePower = (1d-1)^2;

    case 6 % Intralipid 20
      %z0 = 0.5;
      zR = 0.1059;
      dx = 0.00502;
      imgDepth = 2.57;
      numPix = 512;
      datafile = [ dataDir, slash, 'Intralipid_20.cvol' ];
      z0 = 1.6;
      cols2Del = [1:100];
      rows2Del = [301:numPix];
      alpha_R = 2;
      n = 1.353;
      noisePower = (1d-1)^2;

    case 7 % Layered 20140519
      z0 = 1;
      zR = 0.105905; %(mm)
      dx = 0.00502;
      imgDepth = 2.358;
      numPix = 512;
      datafile = [ dataDir, slash, 'multilayeredPDMSphantom.cvol' ];
      rows2Del = 1:50;
      alpha_R = 2;
      n = 1.4;
      noisePower = (0.5d-2)^2;

    case 8 % Porcine Bladder
      z0 = 1;
      zR = 0.105905; %(mm)
      dx = 0.00502;
      imgDepth = 2.358;
      numPix = 512;
      datafile = [ dataDir, slash, 'porcineBladder.cvol' ];
      alpha_R = 2;
      n = 1.37;
      noisePower = (0.5d-2)^2;

    case 9  % Porcine Colon
      z0 = 0.8;
      zR = 0.1059;
      dx = 0.00502;
      imgDepth = 2.358;
      numPix = 512;
      datafile = [ dataDir, slash, 'porcineColon.cvol' ];
      alpha_R = 2;
      n = 1.37;
      noisePower = (0.5d-2)^2;

    case 10 % Porcine Sclera
      z0 = 1;
      zR = 0.105905; %(mm)
      dx = 0.00502;
      imgDepth = 2.358;
      numPix = 512;
      datafile = [ dataDir, slash, 'porcineSclera.cvol' ];
      alpha_R = 2;
      n = 1.37;
      noisePower = (0.5d-2)^2;

    case 11 % Porcine Skin
      z0 = 1;
      zR = 0.105905; %(mm)
      dx = 0.00502;
      imgDepth = 2.358;
      numPix = 512;
      datafile = [ dataDir, slash, 'porcineSkin.cvol' ];
      cols2Del = [];
      rows2Del = [];
      alpha_R = 2;
      n = 1.37;
      noisePower = (0.5d-2)^2;

    otherwise
      error('Invalid data case');
  end

  z = linspace(0, imgDepth, numPix)';

  bScan = load_cvol(datafile);
  I = 10.^(bScan./10); % Convert from db
  
  I(rows2Del, :) = [];
  z(rows2Del, :) = [];
  if numel(rows2Del) > 0 && min(rows2Del) == 1
    zRemove = z(max(rows2Del));
    z0 = z0 - zRemove;
    z = z - zRemove;
  end
  I(:, cols2Del) = [];

  if numel(trueMu)>0
    trueMu(rows2Del, :) = [];
    trueMu(:,cols2Del) = [];
  end

  if dataCase ~= 0
    I = I ./ 55743;     % Divide by this amount to make consistent with
                        % theoretical falloff function
    zR = alpha_R*n*zR;  % Multiply by alpha and n to convert from zR
                        % to apparent zR. See Faber paper for more details
  end

end
