% This software is offered under the GNU General Public License 3.0.  It 
% is offered without any warranty expressed or implied, including the 
% implied warranties of merchantability or fitness for a particular 
% purpose.

% Written by Nicholas Dwork (ndwork@stanford.edu) and 
% Gennifer Smith (gsmith9@stanford.edu).


function muFit = muFit2D_DRC_iwhTV( I, z, z0, zR, ...
  lambda, deltaLambda, dLambda, noisePower, mask )
  % This function requires CVX, which can be gotten from 
  % www.cvxr.com

  eta = 6;    % good value
  epsilon = 1d-3;

  [M, N] = size( I );

  muHat = muFit2D_DRC( I, z, z0, zR, lambda, deltaLambda, dLambda, ...
    noisePower );

  weight = 1 ./ ( I + epsilon );
  
  Dx = -eye(N);
  for i=1:N-1
    Dx(i,i+1) = 1;
  end
  Dx(N,:) = 0;

  muFit = zeros( M, N );
  for i=1:M
    if mod(i,50)==0
      disp(['Working on row ', num2str(i), ' of ', num2str(M)]);
    end
    
    muHatRow = muHat(i,:)';
    weightRow = weight(i,:)';
    maskRow = mask(i,:)';

    if max( maskRow ) == 0
      muFit(i,:) = 0;
    else

      Nx = sum(maskRow);

      wmRow = muHatRow .* weightRow;
      cvx_begin quiet
        variable muRow(N,1)
        minimize 0.5 * sum(( muRow - muHatRow ).^2) + ...
          eta/Nx * norm( wmRow .* ( Dx * muRow ), 1 )
        subject to
          muRow >= 0
      cvx_end

      muFit(i,:) = muRow;
      
    end
  end

end

