% This software is offered under the GNU General Public License 3.0.  It 
% is offered without any warranty expressed or implied, including the 
% implied warranties of merchantability or fitness for a particular 
% purpose.

% Written by Nicholas Dwork (ndwork@stanford.edu) and 
% Gennifer Smith (gsmith9@stanford.edu).

function runMuFit2D
  clear; close all; rng(1);
  addpath(genpath(pwd));

  muAlpha = 0;    % Note, if we know true value than problem is better

  dataDir = './2015_octData';

  dataCase = 0;
  [I, z, dx, z0, zR, muAlpha, muBeta, muL0, lambda, deltaLambda, ...
    dLambda, noisePower, trueMu ] = loadOctData( dataCase, false, dataDir );

  if dataCase == 0
    mask = ones( size(I) );
  else
    mask = findNonZeroMus(I);
  end
  I = I .* mask;

  tic;
  %muFit = muFit2D_ver( I, z );
  muFit = muFit2D_DRC( I, z, z0, zR, lambda, deltaLambda, dLambda, ...
    noisePower );
  %muFit = muFit2D_DRC_gBlur( I, z, z0, zR, lambda, deltaLambda, ...
  %  dLambda, noisePower );
  %muFit = muFit2D_DRC_TV( I, z, z0, zR, lambda, deltaLambda, ...
  %  dLambda, noisePower );
  %muFit = muFit2D_DRC_iwhTV( I, z, z0, zR, lambda, deltaLambda, ...
  %  dLambda, noisePower, mask );
  timeTaken = toc;
  disp(['Time taken (s):', num2str(timeTaken)]);

  if dataCase >= 1 && dataCase <= 6
    avgDepth = 0.25;
    dz = z(2)-z(1);
    [~, N] = size(I);
    midCol = floor(N/2);
    skinLocs = zeros(1,N);
    for i=1:N, skinLocs(i) = find( mask(:,i)==1, 1 ); end;
    depthMM = skinLocs + floor(avgDepth/dz);
    meanI = mean(I,2);
    midSkinLoc = skinLocs(midCol);
    meanFit = mean( muFit(midSkinLoc:depthMM,midCol) );
    disp(['MuFit: ', num2str( meanFit ) ]);
    medMuFit = median( muFit, 2 );
    medFit = mean( medMuFit(midSkinLoc:depthMM) );
    disp(['MuFit Median: ', num2str( medFit ) ]);

    avgMus = muFit(skinLocs:depthMM,:);
    avgMus = mean( avgMus, 1 );
    disp(['Mean Mu for all cols: ', num2str(mean(avgMus)) ]);
    disp(['Med Mu for all cols: ', num2str(median(avgMus)) ]);
    disp(['StdDev for all cols: ', num2str(std(avgMus)) ]);
    disp(['Max Mu for all cols: ', num2str(max(avgMus)) ]);
    disp(['Min Mu for all cols: ', num2str(min(avgMus)) ]);
  end

  figure, imshow( muFit, [0 3.0] );

  if dataCase == 0
    figure; plot( z, trueMu(:,50), 'k' ); ylim([0 5]);
    hold on; plot( z, muFit(:,50), 'b' );
    xlabel('Depth (mm)'); ylabel('Attenuation Coefficient (mm^{-1})');
    legend('truth','estimate');
    [meanEtbDepth, meanVDepth] = findErrorMetrics(muFit,trueMu,z);
    disp(['Mean ETB Depth: ', num2str(meanEtbDepth)]);
    disp(['Mean V Depth: ', num2str(meanVDepth)]);
  end

end

